//=============================================================================
// (c) Copyright 2005 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MeasureReferences.c  v5.9
// Desc: Sample program that demonstrates how to measure and set the reference
//       voltages in the EEPROM of your board
//	Created by	KL
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif
#include <windows.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "../../../../current/dev/source/dscud.h"
#endif

// Linux and QNX
#if defined(linux) || defined(__QNXNTO__) || defined(_WRS_VXWORKS_5_X)
#include <stdlib.h>
#include <math.h>
#include <time.h>
// diamond driver includes
#include "dscud.h"


#ifdef _WRS_VXWORKS_5_X
#include <selectLib.h>
#define main DMM32XMeasureReferences
#else
#include <sys/time.h>
#endif

static int kbhit()
{
	struct timeval timeout;
	fd_set rfds;

	timeout.tv_sec = 0;
	timeout.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(0, &rfds);

	if ( select(0+1, &rfds, NULL, NULL, &timeout) > 0 )
		return getchar();

	return 0;
}
#endif


// var declarations
BYTE result;  // returned error code
DSCB dscb;    // handle used to refer to the board
DSCCB dsccb;  // structure containing board settings
DSCAUTOCAL dscautocal; // structure containing auto-calibration settings
ERRPARAMS errorParams; // structure for returning error code and error string
int intBuff;  // temp variable of size int
char menu;

int i;	// miscellaneous counter

DFLOAT refs[7];

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. Measure References
//	    IV. Cleanup
//
//=============================================================================

int main( void )
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//=========================================================================

	if( ( result = dscInit( DSC_VERSION ) ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );

		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the DMM-32X-AT board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	printf( "\nDMM32X BOARD INITIALIZATION:\n" );

	printf("Enter the base address (default 0x300) : ");
	scanf("%hx", &dsccb.base_address );	

	dsccb.int_level = 3;

	if(dscInitBoard(DSC_DMM32X, &dsccb, &dscb)!= DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	printf("\n");
	getchar();

	//=========================================================================
	// III. MEASURE REFERENCES
	//
	//		Measure the reference voltages for your board and then mention how
	//		to store these values in the EEPROM. 
	//
	//=========================================================================

	do {

		printf("\nSelect from the following menu \n");
		printf("0) Exit\n");
		printf("1) EEProm dump \n");
		printf("2) Measure reference voltage \n");

		scanf("%c", &menu);

		if( menu == '0') 
			break;
		else if( menu == '1')
		{
			if( dscGetReferenceVoltages( dscb, refs) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf( stderr, "dscGetReferenceVoltage error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
			}
	
			for( i = 0; i < 6 ; i++)
			{
				printf("The Stored Reference Voltage %lf\n", refs[i]);
			}
		}
		else if( menu == '2')
		{
		
			printf( "Locate J9 on the DMM32X. Place the ground probe of your voltmeter\non the negative pin and the other probe \non the positive pin\n\n" );
	
			dscSetCalMux(dscb, 1);

			for( i = 0; i < 6; i++ )
			{
				if( dscADSetChannel(dscb, (unsigned char) i, (unsigned char)i) != DE_NONE )
				{
					dscGetLastError(&errorParams);
					fprintf( stderr, "dscADsetChannel error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
					return 0;
				}
		
				printf( "Enter the voltage on the voltmeter (cal mux channel %d): ", i );
				scanf( "%lf", &refs[i] );
			}

			dscSetCalMux(dscb, 0);

			if( dscSetReferenceVoltages( dscb, refs) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf( stderr, "dscSetReferenceVoltages error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
			}

			for( i =0; i < 6; i++)
				refs[i] =0;
	
			if( dscGetReferenceVoltages( dscb, refs) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf( stderr, "dscGetReferenceVoltages error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
			}

			for( i = 0; i < 6 ; i++)
			{
				printf("The Stored Reference Voltage %lf\n", refs[i]);
			}

			printf( "Complete!\n" );
		}
		getchar();
	} while (1);
	//=========================================================================
	// IV. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree();

	printf( "\nMeasureReferences completed. \n" );

	return 0;
} // end main()



#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif

